@@ -16,6 +16,7 @@
 #include <qpdf/Pl_DCT.hh>
 #include <qpdf/QIntC.hh>
 #include <iostream>
+#include <memory>
 #include <string.h>
 #include <stdlib.h>
 
@@ -30,7 +31,7 @@ class ImageProvider: public QPDFObjectHandle::StreamDataProvider
                   std::string const& filter);
     virtual ~ImageProvider();
     virtual void provideStreamData(int objid, int generation,
-				   Pipeline* pipeline);
+                                   Pipeline* pipeline);
     size_t getWidth() const;
     size_t getHeight() const;
 
@@ -105,22 +106,25 @@ void
 ImageProvider::provideStreamData(int objid, int generation,
                                  Pipeline* pipeline)
 {
-    std::vector<PointerHolder<Pipeline> > to_delete;
+    std::vector<std::shared_ptr<Pipeline>> to_delete;
     Pipeline* p = pipeline;
+    std::shared_ptr<Pipeline> p_new;
 
     if (filter == "/DCTDecode")
     {
-        p = new Pl_DCT(
+        p_new = std::make_shared<Pl_DCT>(
             "image encoder", pipeline,
             QIntC::to_uint(width), QIntC::to_uint(getHeight()),
             QIntC::to_int(stripes[0].length()), j_color_space);
-        to_delete.push_back(p);
+        to_delete.push_back(p_new);
+        p = p_new.get();
     }
     else if (filter == "/RunLengthDecode")
     {
-        p = new Pl_RunLength(
+        p_new = std::make_shared<Pl_RunLength>(
             "image encoder", pipeline, Pl_RunLength::a_encode);
-        to_delete.push_back(p);
+        to_delete.push_back(p_new);
+        p = p_new.get();
     }
 
     for (size_t i = 0; i < n_stripes; ++i)
@@ -138,7 +142,7 @@ ImageProvider::provideStreamData(int objid, int generation,
 void usage()
 {
     std::cerr << "Usage: " << whoami << " filename" << std::endl
-	      << "Creates a simple PDF and writes it to filename" << std::endl;
+              << "Creates a simple PDF and writes it to filename" << std::endl;
     exit(2);
 }
 
@@ -178,16 +182,15 @@ void add_page(QPDFPageDocumentHelper& dh, QPDFObjectHandle font,
     size_t width = p->getWidth();
     size_t height = p->getHeight();
     QPDFObjectHandle image = QPDFObjectHandle::newStream(&pdf);
-    image.replaceDict(QPDFObjectHandle::parse(
-                          "<<"
-                          " /Type /XObject"
-                          " /Subtype /Image"
-                          " /BitsPerComponent 8"
-                          ">>"));
-    QPDFObjectHandle image_dict = image.getDict();
+    auto image_dict = "<<"
+                      " /Type /XObject"
+                      " /Subtype /Image"
+                      " /BitsPerComponent 8"
+                      ">>"_qpdf;
     image_dict.replaceKey("/ColorSpace", newName(color_space));
     image_dict.replaceKey("/Width", newInteger(width));
     image_dict.replaceKey("/Height", newInteger(height));
+    image.replaceDict(image_dict);
 
     // Provide the stream data.
     image.replaceStreamData(provider,
@@ -195,8 +198,7 @@ void add_page(QPDFPageDocumentHelper& dh, QPDFObjectHandle font,
                             QPDFObjectHandle::newNull());
 
     // Create direct objects as needed by the page dictionary.
-    QPDFObjectHandle procset = QPDFObjectHandle::parse(
-        "[/PDF /Text /ImageC]");
+    QPDFObjectHandle procset = "[/PDF /Text /ImageC]"_qpdf;
 
     QPDFObjectHandle rfont = QPDFObjectHandle::newDictionary();
     rfont.replaceKey("/F1", font);
@@ -209,21 +211,16 @@ void add_page(QPDFPageDocumentHelper& dh, QPDFObjectHandle font,
     resources.replaceKey("/Font", rfont);
     resources.replaceKey("/XObject", xobject);
 
-    QPDFObjectHandle mediabox = QPDFObjectHandle::newArray();
-    mediabox.appendItem(newInteger(0));
-    mediabox.appendItem(newInteger(0));
-    mediabox.appendItem(newInteger(612));
-    mediabox.appendItem(newInteger(392));
-
     // Create the page content stream
     QPDFObjectHandle contents = createPageContents(
         pdf, color_space + " with filter " + filter);
 
     // Create the page dictionary
     QPDFObjectHandle page = pdf.makeIndirectObject(
-        QPDFObjectHandle::newDictionary());
-    page.replaceKey("/Type", newName("/Page"));
-    page.replaceKey("/MediaBox", mediabox);
+        "<<"
+        " /Type /Page"
+        " /MediaBox [0 0 612 392]"
+        ">>"_qpdf);
     page.replaceKey("/Contents", contents);
     page.replaceKey("/Resources", resources);
 
@@ -255,20 +252,16 @@ static void check(char const* filename,
 
     QPDF pdf;
     pdf.processFile(filename);
-    QPDFPageDocumentHelper dh(pdf);
-    std::vector<QPDFPageObjectHelper> pages = dh.getAllPages();
+    auto pages = QPDFPageDocumentHelper(pdf).getAllPages();
     if (n_color_spaces * n_filters != pages.size())
     {
         throw std::logic_error("incorrect number of pages");
     }
     size_t pageno = 1;
     bool errors = false;
-    for (std::vector<QPDFPageObjectHelper>::iterator page_iter =
-             pages.begin();
-         page_iter != pages.end(); ++page_iter)
+    for (auto& page : pages)
     {
-        QPDFPageObjectHelper& page(*page_iter);
-        std::map<std::string, QPDFObjectHandle> images = page.getImages();
+        auto images = page.getImages();
         if (images.size() != 1)
         {
             throw std::logic_error("incorrect number of images on page");
@@ -277,8 +270,7 @@ static void check(char const* filename,
         // Check filter and color space.
         std::string desired_color_space =
             color_spaces[(pageno - 1) / n_color_spaces];
-        std::string desired_filter =
-            filters[(pageno - 1) % n_filters];
+        std::string desired_filter = filters[(pageno - 1) % n_filters];
         // In the default mode, QPDFWriter will compress with
         // /FlateDecode if no filters are provided.
         if (desired_filter == "null")
@@ -290,15 +282,14 @@ static void check(char const* filename,
         QPDFObjectHandle color_space = image_dict.getKey("/ColorSpace");
         QPDFObjectHandle filter = image_dict.getKey("/Filter");
         bool this_errors = false;
-        if (! (filter.isName() && (filter.getName() == desired_filter)))
+        if (! filter.isNameAndEquals(desired_filter))
         {
             this_errors = errors = true;
             std::cout << "page " << pageno << ": expected filter "
                       << desired_filter << "; actual filter = "
                       << filter.unparse() << std::endl;
         }
-        if (! (color_space.isName() &&
-               (color_space.getName() == desired_color_space)))
+        if (! color_space.isNameAndEquals(desired_color_space))
         {
             this_errors = errors = true;
             std::cout << "page " << pageno << ": expected color space "
@@ -309,7 +300,7 @@ static void check(char const* filename,
         if (! this_errors)
         {
             // Check image data
-            PointerHolder<Buffer> actual_data =
+            auto actual_data =
                 image.getStreamData(qpdf_dl_all);
             ImageProvider* p = new ImageProvider(desired_color_space, "null");
             PointerHolder<QPDFObjectHandle::StreamDataProvider> provider(p);
@@ -381,14 +372,13 @@ static void create_pdf(char const* filename)
     // Add an indirect object to contain a font descriptor for the
     // built-in Helvetica font.
     QPDFObjectHandle font = pdf.makeIndirectObject(
-        QPDFObjectHandle::parse(
-            "<<"
-            " /Type /Font"
-            " /Subtype /Type1"
-            " /Name /F1"
-            " /BaseFont /Helvetica"
-            " /Encoding /WinAnsiEncoding"
-            ">>"));
+        "<<"
+        " /Type /Font"
+        " /Subtype /Type1"
+        " /Name /F1"
+        " /BaseFont /Helvetica"
+        " /Encoding /WinAnsiEncoding"
+        ">>"_qpdf);
 
     std::vector<std::string> color_spaces;
     color_spaces.push_back("/DeviceCMYK");
@@ -399,13 +389,11 @@ static void create_pdf(char const* filename)
     filters.push_back("/DCTDecode");
     filters.push_back("/RunLengthDecode");
     QPDFPageDocumentHelper dh(pdf);
-    for (std::vector<std::string>::iterator c_iter = color_spaces.begin();
-         c_iter != color_spaces.end(); ++c_iter)
+    for (auto const& color_space : color_spaces)
     {
-        for (std::vector<std::string>::iterator f_iter = filters.begin();
-             f_iter != filters.end(); ++f_iter)
+        for (auto const& filter : filters)
         {
-            add_page(dh, font, *c_iter, *f_iter);
+            add_page(dh, font, color_space, filter);
         }
     }
 
@@ -424,22 +412,22 @@ int main(int argc, char* argv[])
     // For libtool's sake....
     if (strncmp(whoami, "lt-", 3) == 0)
     {
-	whoami += 3;
+        whoami += 3;
     }
     if (argc != 2)
     {
-	usage();
+        usage();
     }
     char const* filename = argv[1];
 
     try
     {
-	create_pdf(filename);
+        create_pdf(filename);
     }
     catch (std::exception& e)
     {
-	std::cerr << e.what() << std::endl;
-	exit(2);
+        std::cerr << e.what() << std::endl;
+        exit(2);
     }
 
     return 0;
