@@ -14,7 +14,7 @@
 // uses the pattern of having the stream data provider class use a
 // second QPDF instance with copies of streams from the original QPDF
 // so that the stream data provider can access the original stream
-// data. This is implement very efficiently inside the qpdf library as
+// data. This is implemented very efficiently inside the qpdf library as
 // the second QPDF instance knows how to read the stream data from the
 // original input file, so no extra copies of the original stream data
 // are made.
@@ -196,10 +196,9 @@ class StreamReplacer: public QPDFObjectHandle::StreamDataProvider
     //   pipeline and passes nullptr to dict_updates. In this mode,
     //   the stream dictionary has already been altered, and the
     //   original stream data is no longer directly accessible. Trying
-    //   to retrieve the stream data would be an infinite loop because
+    //   to retrieve the stream data would cause an infinite loop because
     //   it would just end up calling provideStreamData again. This is
-    //   why maybeReplace uses a stashed copy of the original stream
-    //   from the "other" QPDF object.
+    //   why maybeReplace uses a stashed copy of the original stream.
 
     // Additional explanation can be found in the method
     // implementations.
@@ -208,7 +207,7 @@ class StreamReplacer: public QPDFObjectHandle::StreamDataProvider
     StreamReplacer(QPDF* pdf);
     virtual ~StreamReplacer() = default;
     virtual void provideStreamData(int objid, int generation,
-				   Pipeline* pipeline) override;
+                                   Pipeline* pipeline) override;
 
     void registerStream(
         QPDFObjectHandle stream,
@@ -223,11 +222,6 @@ class StreamReplacer: public QPDFObjectHandle::StreamDataProvider
     // we are replacing. We need this to create a new stream.
     QPDF* pdf;
 
-    // This second QPDF instance gives us a place to copy streams to
-    // so that we can access the original stream data of the streams
-    // whose data we are replacing.
-    QPDF other;
-
     // Map the object/generation in original file to the copied stream
     // in "other". We use this to retrieve the original data.
     std::map<QPDFObjGen, QPDFObjectHandle> copied_streams;
@@ -241,10 +235,6 @@ class StreamReplacer: public QPDFObjectHandle::StreamDataProvider
 StreamReplacer::StreamReplacer(QPDF* pdf) :
     pdf(pdf)
 {
-    // Our "other" QPDF is just a place to stash streams. It doesn't
-    // have to be a valid PDF with pages, etc. We are never going to
-    // write this out.
-    this->other.emptyPDF();
 }
 
 bool
@@ -375,9 +365,11 @@ StreamReplacer::registerStream(
 
     if (should_replace)
     {
-        // Copy the stream to another QPDF object so we can get to the
-        // original data from the stream data provider.
-        this->copied_streams[og] = this->other.copyForeignObject(stream);
+        // Copy the stream so we can get to the original data from the
+        // stream data provider. This doesn't actually copy any data,
+        // but the copy retains the original stream data after the
+        // original one is modified.
+        this->copied_streams[og] = stream.copyStream();
         // Update the stream dictionary with any changes.
         auto dict = stream.getDict();
         for (auto const& k: dict_updates.getKeys())
@@ -469,7 +461,7 @@ static void usage()
 {
     std::cerr
         << "\n"
-        << "Usage: " << whoami << " [ --decode-specialized ] infile outfile\n"
+        << "Usage: " << whoami << " [--decode-specialized] infile outfile\n"
         << std::endl;
     exit(2);
 }
@@ -481,7 +473,7 @@ int main(int argc, char* argv[])
     // For libtool's sake....
     if (strncmp(whoami, "lt-", 3) == 0)
     {
-	whoami += 3;
+        whoami += 3;
     }
 
     char const* infilename = 0;
@@ -525,7 +517,7 @@ int main(int argc, char* argv[])
     catch (std::exception &e)
     {
         std::cerr << whoami << ": exception: " << e.what() << std::endl;
-	exit(2);
+        exit(2);
     }
 
     return 0;
