@@ -1,8 +1,8 @@
 #include <qpdf/BitWriter.hh>
 
-// See comments in bits.cc
+// See comments in bits_functions.hh
 #define BITS_WRITE 1
-#include "bits.icc"
+#include <qpdf/bits_functions.hh>
 
 BitWriter::BitWriter(Pipeline* pl) :
     pl(pl),
@@ -43,7 +43,7 @@ BitWriter::flush()
 {
     if (bit_offset < 7)
     {
-	size_t bits_to_write = bit_offset + 1;
-	write_bits(this->ch, this->bit_offset, 0, bits_to_write, this->pl);
+        size_t bits_to_write = bit_offset + 1;
+        write_bits(this->ch, this->bit_offset, 0, bits_to_write, this->pl);
     }
 }
