@@ -27,12 +27,17 @@ InsecureRandomDataProvider::random()
 {
     if (! this->seeded_random)
     {
-	// Seed the random number generator with something simple, but
-	// just to be interesting, don't use the unmodified current
-	// time.  It would be better if this were a more secure seed.
-        QUtil::srandom(static_cast<unsigned int>(
-                           QUtil::get_current_time() ^ 0xcccc));
-	this->seeded_random = true;
+        // Seed the random number generator with something simple, but
+        // just to be interesting, don't use the unmodified current
+        // time.  It would be better if this were a more secure seed.
+        unsigned int seed = static_cast<unsigned int>(
+            QUtil::get_current_time() ^ 0xcccc);
+#ifdef HAVE_RANDOM
+        ::srandom(seed);
+#else
+        srand(seed);
+#endif
+        this->seeded_random = true;
     }
 
 #  ifdef HAVE_RANDOM
