@@ -1,8 +1,10 @@
 #include <qpdf/Pl_Buffer.hh>
+
 #include <stdexcept>
 #include <algorithm>
 #include <assert.h>
 #include <string.h>
+#include <stdlib.h>
 
 Pl_Buffer::Members::Members() :
     ready(true),
@@ -27,16 +29,16 @@ Pl_Buffer::~Pl_Buffer()
 void
 Pl_Buffer::write(unsigned char* buf, size_t len)
 {
-    if (this->m->data.getPointer() == 0)
+    if (this->m->data.get() == 0)
     {
-        this->m->data = new Buffer(len);
+        this->m->data = make_pointer_holder<Buffer>(len);
     }
     size_t cur_size = this->m->data->getSize();
     size_t left = cur_size - this->m->total_size;
     if (left < len)
     {
         size_t new_size = std::max(this->m->total_size + len, 2 * cur_size);
-        PointerHolder<Buffer> b = new Buffer(new_size);
+        auto b = make_pointer_holder<Buffer>(new_size);
         memcpy(b->getBuffer(), this->m->data->getBuffer(), this->m->total_size);
         this->m->data = b;
     }
@@ -49,7 +51,7 @@ Pl_Buffer::write(unsigned char* buf, size_t len)
 
     if (getNext(true))
     {
-	getNext()->write(buf, len);
+        getNext()->write(buf, len);
     }
 }
 
@@ -59,7 +61,7 @@ Pl_Buffer::finish()
     this->m->ready = true;
     if (getNext(true))
     {
-	getNext()->finish();
+        getNext()->finish();
     }
 }
 
@@ -68,7 +70,7 @@ Pl_Buffer::getBuffer()
 {
     if (! this->m->ready)
     {
-	throw std::logic_error("Pl_Buffer::getBuffer() called when not ready");
+        throw std::logic_error("Pl_Buffer::getBuffer() called when not ready");
     }
 
     Buffer* b = new Buffer(this->m->total_size);
@@ -77,6 +79,34 @@ Pl_Buffer::getBuffer()
         unsigned char* p = b->getBuffer();
         memcpy(p, this->m->data->getBuffer(), this->m->total_size);
     }
-    this->m = new Members();
+    this->m = PointerHolder<Members>(new Members());
     return b;
 }
+
+PointerHolder<Buffer>
+Pl_Buffer::getBufferSharedPointer()
+{
+    return PointerHolder<Buffer>(getBuffer());
+}
+
+void
+Pl_Buffer::getMallocBuffer(unsigned char **buf, size_t* len)
+{
+    if (! this->m->ready)
+    {
+        throw std::logic_error(
+            "Pl_Buffer::getMallocBuffer() called when not ready");
+    }
+
+    *len = this->m->total_size;
+    if (this->m->total_size > 0)
+    {
+        *buf = reinterpret_cast<unsigned char*>(malloc(this->m->total_size));
+        memcpy(*buf, this->m->data->getBuffer(), this->m->total_size);
+    }
+    else
+    {
+        *buf = nullptr;
+    }
+    this->m = PointerHolder<Members>(new Members());
+}
