@@ -1,5 +1,8 @@
 #include <qpdf/Pl_PNGFilter.hh>
+
 #include <qpdf/QTC.hh>
+#include <qpdf/QUtil.hh>
+
 #include <stdexcept>
 #include <string.h>
 #include <limits.h>
@@ -10,7 +13,7 @@ static int abs_diff(int a, int b)
 }
 
 Pl_PNGFilter::Pl_PNGFilter(char const* identifier, Pipeline* next,
-			   action_e action, unsigned int columns,
+                           action_e action, unsigned int columns,
                            unsigned int samples_per_pixel,
                            unsigned int bits_per_sample) :
     Pipeline(identifier, next),
@@ -45,14 +48,14 @@ Pl_PNGFilter::Pl_PNGFilter(char const* identifier, Pipeline* next,
             "PNGFilter created with invalid columns value");
     }
     this->bytes_per_row = bpr & UINT_MAX;
-    this->buf1 = PointerHolder<unsigned char>(
-        true, new unsigned char[this->bytes_per_row + 1]);
-    this->buf2 = PointerHolder<unsigned char>(
-        true, new unsigned char[this->bytes_per_row + 1]);
-    memset(this->buf1.getPointer(), 0, this->bytes_per_row + 1);
-    memset(this->buf2.getPointer(), 0, this->bytes_per_row + 1);
-    this->cur_row = this->buf1.getPointer();
-    this->prev_row = this->buf2.getPointer();
+    this->buf1 = make_array_pointer_holder<unsigned char>(
+        this->bytes_per_row + 1);
+    this->buf2 = make_array_pointer_holder<unsigned char>(
+        this->bytes_per_row + 1);
+    memset(this->buf1.get(), 0, this->bytes_per_row + 1);
+    memset(this->buf2.get(), 0, this->bytes_per_row + 1);
+    this->cur_row = this->buf1.get();
+    this->prev_row = this->buf2.get();
 
     // number of bytes per incoming row
     this->incoming = (action == a_encode ?
@@ -71,24 +74,24 @@ Pl_PNGFilter::write(unsigned char* data, size_t len)
     size_t offset = 0;
     while (len >= left)
     {
-	// finish off current row
-	memcpy(this->cur_row + this->pos, data + offset, left);
-	offset += left;
-	len -= left;
+        // finish off current row
+        memcpy(this->cur_row + this->pos, data + offset, left);
+        offset += left;
+        len -= left;
 
-	processRow();
+        processRow();
 
-	// Swap rows
-	unsigned char* t = this->prev_row;
-	this->prev_row = this->cur_row;
-	this->cur_row = t ? t : this->buf2.getPointer();
-	memset(this->cur_row, 0, this->bytes_per_row + 1);
-	left = this->incoming;
-	this->pos = 0;
+        // Swap rows
+        unsigned char* t = this->prev_row;
+        this->prev_row = this->cur_row;
+        this->cur_row = t ? t : this->buf2.get();
+        memset(this->cur_row, 0, this->bytes_per_row + 1);
+        left = this->incoming;
+        this->pos = 0;
     }
     if (len)
     {
-	memcpy(this->cur_row + this->pos, data + offset, len);
+        memcpy(this->cur_row + this->pos, data + offset, len);
     }
     this->pos += len;
 }
@@ -98,11 +101,11 @@ Pl_PNGFilter::processRow()
 {
     if (this->action == a_encode)
     {
-	encodeRow();
+        encodeRow();
     }
     else
     {
-	decodeRow();
+        decodeRow();
     }
 }
 
@@ -247,16 +250,16 @@ Pl_PNGFilter::encodeRow()
     getNext()->write(&ch, 1);
     if (this->prev_row)
     {
-	for (unsigned int i = 0; i < this->bytes_per_row; ++i)
-	{
-	    ch = static_cast<unsigned char>(
+        for (unsigned int i = 0; i < this->bytes_per_row; ++i)
+        {
+            ch = static_cast<unsigned char>(
                 this->cur_row[i] - this->prev_row[i]);
-	    getNext()->write(&ch, 1);
-	}
+            getNext()->write(&ch, 1);
+        }
     }
     else
     {
-	getNext()->write(this->cur_row, this->bytes_per_row);
+        getNext()->write(this->cur_row, this->bytes_per_row);
     }
 }
 
@@ -265,11 +268,11 @@ Pl_PNGFilter::finish()
 {
     if (this->pos)
     {
-	// write partial row
-	processRow();
+        // write partial row
+        processRow();
     }
     this->prev_row = 0;
-    this->cur_row = buf1.getPointer();
+    this->cur_row = buf1.get();
     this->pos = 0;
     memset(this->cur_row, 0, this->bytes_per_row + 1);
 
