@@ -280,9 +280,7 @@ QPDFAcroFormDocumentHelper::getFieldForAnnotation(QPDFAnnotationObjectHelper h)
 {
     QPDFObjectHandle oh = h.getObjectHandle();
     QPDFFormFieldObjectHelper result(QPDFObjectHandle::newNull());
-    if (! (oh.isDictionary() &&
-           oh.getKey("/Subtype").isName() &&
-           (oh.getKey("/Subtype").getName() == "/Widget")))
+    if (! oh.isDictionaryOfType("", "/Widget"))
     {
         return result;
     }
@@ -770,7 +768,7 @@ QPDFAcroFormDocumentHelper::adjustDefaultAppearances(
     ResourceReplacer rr(dr_map, rf.getNamesByResourceType());
     Pl_Buffer buf_pl("filtered DA");
     da_stream.filterAsContents(&rr, &buf_pl);
-    PointerHolder<Buffer> buf = buf_pl.getBuffer();
+    auto buf = buf_pl.getBufferSharedPointer();
     std::string new_da(
         reinterpret_cast<char*>(buf->getBuffer()), buf->getSize());
     obj.replaceKey("/DA", QPDFObjectHandle::newString(new_da));
@@ -873,7 +871,7 @@ QPDFAcroFormDocumentHelper::adjustAppearanceStream(
             QTC::TC("qpdf", "QPDFAcroFormDocumentHelper AP parse error");
         }
         auto rr = new ResourceReplacer(dr_map, rf.getNamesByResourceType());
-        PointerHolder<QPDFObjectHandle::TokenFilter> tf = rr;
+        auto tf = PointerHolder<QPDFObjectHandle::TokenFilter>(rr);
         stream.addTokenFilter(tf);
     }
     catch (std::exception& e)
@@ -904,8 +902,8 @@ QPDFAcroFormDocumentHelper::transformAnnotations(
     }
     else if ((from_qpdf != &this->qpdf) && (! from_afdh))
     {
-        afdhph = new QPDFAcroFormDocumentHelper(*from_qpdf);
-        from_afdh = afdhph.getPointer();
+        afdhph = make_pointer_holder<QPDFAcroFormDocumentHelper>(*from_qpdf);
+        from_afdh = afdhph.get();
     }
     bool foreign = (from_qpdf != &this->qpdf);
 
@@ -947,8 +945,12 @@ QPDFAcroFormDocumentHelper::transformAnnotations(
         {
             if (from_acroform.getKey("/DR").isDictionary())
             {
-                from_dr = this->qpdf.copyForeignObject(
-                    from_acroform.getKey("/DR"));
+                from_dr = from_acroform.getKey("/DR");
+                if (! from_dr.isIndirect())
+                {
+                    from_dr = from_qpdf->makeIndirectObject(from_dr);
+                }
+                from_dr = this->qpdf.copyForeignObject(from_dr);
             }
             if (from_acroform.getKey("/DA").isString())
             {
@@ -998,8 +1000,10 @@ QPDFAcroFormDocumentHelper::transformAnnotations(
             }
             // Merge the other document's /DR, creating a conflict
             // map. mergeResources checks to make sure both objects
-            // are dictionaries.
-            from_dr.makeResourcesIndirect(*from_qpdf);
+            // are dictionaries. By this point, if this is foreign,
+            // from_dr has been copied, so we use the target qpdf as
+            // the owning qpdf.
+            from_dr.makeResourcesIndirect(this->qpdf);
             dr.mergeResources(from_dr, &dr_map);
 
             if (from_afdh->getNeedAppearances())
@@ -1105,6 +1109,7 @@ QPDFAcroFormDocumentHelper::transformAnnotations(
                 // They will already be copied, so we'll get the right
                 // object back.
 
+                // top_field and ffield_oh are known to be indirect.
                 top_field = this->qpdf.copyForeignObject(top_field);
                 ffield_oh = this->qpdf.copyForeignObject(ffield_oh);
             }
@@ -1216,6 +1221,10 @@ QPDFAcroFormDocumentHelper::transformAnnotations(
         }
         if (foreign)
         {
+            if (! annot.isIndirect())
+            {
+                annot = from_qpdf->makeIndirectObject(annot);
+            }
             annot = this->qpdf.copyForeignObject(annot);
         }
         maybe_copy_object(annot);
