@@ -1,4 +1,5 @@
 #include <qpdf/QPDFAnnotationObjectHelper.hh>
+
 #include <qpdf/QTC.hh>
 #include <qpdf/QPDFMatrix.hh>
 #include <qpdf/QUtil.hh>
@@ -76,12 +77,12 @@ QPDFAnnotationObjectHelper::getAppearanceStream(
             QPDFObjectHandle ap_sub_val = ap_sub.getKey(desired_state);
             if (ap_sub_val.isStream())
             {
-                QTC::TC("qpdf", "QPDFAnnotationObjectHelper AN sub stream");
+                QTC::TC("qpdf", "QPDFAnnotationObjectHelper AP sub stream");
                 return ap_sub_val;
             }
         }
     }
-    QTC::TC("qpdf", "QPDFAnnotationObjectHelper AN null");
+    QTC::TC("qpdf", "QPDFAnnotationObjectHelper AP null");
     return QPDFObjectHandle::newNull();
 }
 
