@@ -1,4 +1,5 @@
 #include <qpdf/QPDFEFStreamObjectHelper.hh>
+
 #include <qpdf/QIntC.hh>
 #include <qpdf/QUtil.hh>
 #include <qpdf/Pl_Count.hh>
@@ -76,7 +77,7 @@ QPDFEFStreamObjectHelper::getSize()
 std::string
 QPDFEFStreamObjectHelper::getSubtype()
 {
-    auto val = getParam("/Subtype");
+    auto val = this->oh.getDict().getKey("/Subtype");
     if (val.isName())
     {
         auto n = val.getName();
@@ -141,7 +142,8 @@ QPDFEFStreamObjectHelper::setModDate(std::string const& date)
 QPDFEFStreamObjectHelper&
 QPDFEFStreamObjectHelper::setSubtype(std::string const& subtype)
 {
-    setParam("/Subtype", QPDFObjectHandle::newName("/" + subtype));
+    this->oh.getDict().replaceKey(
+        "/Subtype", QPDFObjectHandle::newName("/" + subtype));
     return *this;
 }
 
