@@ -1,4 +1,5 @@
 #include <qpdf/QPDFMatrix.hh>
+
 #include <qpdf/QUtil.hh>
 #include <algorithm>
 
@@ -142,3 +143,14 @@ QPDFMatrix::transformRectangle(QPDFObjectHandle::Rectangle r) const
         *std::max_element(tx.begin(), tx.end()),
         *std::max_element(ty.begin(), ty.end()));
 }
+
+bool
+QPDFMatrix::operator==(QPDFMatrix const& rhs) const
+{
+    return ((this->a == rhs.a) &&
+            (this->b == rhs.b) &&
+            (this->c == rhs.c) &&
+            (this->d == rhs.d) &&
+            (this->e == rhs.e) &&
+            (this->f == rhs.f));
+}
