@@ -1,4 +1,5 @@
 #include <qpdf/QPDFSystemError.hh>
+
 #include <qpdf/QUtil.hh>
 #include <string.h>
 
@@ -10,7 +11,7 @@ QPDFSystemError::QPDFSystemError(std::string const& description,
 {
 }
 
-QPDFSystemError::~QPDFSystemError() throw ()
+QPDFSystemError::~QPDFSystemError() noexcept
 {
 }
 
