@@ -1,6 +1,8 @@
 #include <qpdf/QPDFXRefEntry.hh>
+
 #include <qpdf/QPDFExc.hh>
 #include <qpdf/QUtil.hh>
+#include <qpdf/QIntC.hh>
 
 QPDFXRefEntry::QPDFXRefEntry() :
     type(0),
@@ -16,8 +18,8 @@ QPDFXRefEntry::QPDFXRefEntry(int type, qpdf_offset_t field1, int field2) :
 {
     if ((type < 1) || (type > 2))
     {
-	throw std::logic_error(
-	    "invalid xref type " + QUtil::int_to_string(type));
+        throw std::logic_error(
+            "invalid xref type " + QUtil::int_to_string(type));
     }
 }
 
@@ -32,8 +34,8 @@ QPDFXRefEntry::getOffset() const
 {
     if (this->type != 1)
     {
-	throw std::logic_error(
-	    "getOffset called for xref entry of type != 1");
+        throw std::logic_error(
+            "getOffset called for xref entry of type != 1");
     }
     return this->field1;
 }
@@ -43,10 +45,10 @@ QPDFXRefEntry::getObjStreamNumber() const
 {
     if (this->type != 2)
     {
-	throw std::logic_error(
-	    "getObjStreamNumber called for xref entry of type != 2");
+        throw std::logic_error(
+            "getObjStreamNumber called for xref entry of type != 2");
     }
-    return (int) this->field1;
+    return QIntC::to_int(this->field1);
 }
 
 int
@@ -54,8 +56,8 @@ QPDFXRefEntry::getObjStreamIndex() const
 {
     if (this->type != 2)
     {
-	throw std::logic_error(
-	    "getObjStreamIndex called for xref entry of type != 2");
+        throw std::logic_error(
+            "getObjStreamIndex called for xref entry of type != 2");
     }
     return this->field2;
 }
