@@ -86,7 +86,7 @@ QPDF::getAllPages()
             getRoot().replaceKey("/Pages", pages);
         }
         seen.clear();
-	getAllPagesInternal(pages, this->m->all_pages, visited, seen);
+        getAllPagesInternal(pages, this->m->all_pages, visited, seen);
     }
     return this->m->all_pages;
 }
@@ -110,10 +110,10 @@ QPDF::getAllPagesInternal(QPDFObjectHandle cur_node,
     if (cur_node.hasKey("/Kids"))
     {
         wanted_type = "/Pages";
-	QPDFObjectHandle kids = cur_node.getKey("/Kids");
-	int n = kids.getArrayNItems();
-	for (int i = 0; i < n; ++i)
-	{
+        QPDFObjectHandle kids = cur_node.getKey("/Kids");
+        int n = kids.getArrayNItems();
+        for (int i = 0; i < n; ++i)
+        {
             QPDFObjectHandle kid = kids.getArrayItem(i);
             if (! kid.isIndirect())
             {
@@ -129,18 +129,17 @@ QPDF::getAllPagesInternal(QPDFObjectHandle cur_node,
                 kid = makeIndirectObject(QPDFObjectHandle(kid).shallowCopy());
                 kids.setArrayItem(i, kid);
             }
-	    getAllPagesInternal(kid, result, visited, seen);
-	}
+            getAllPagesInternal(kid, result, visited, seen);
+        }
     }
     else
     {
         wanted_type = "/Page";
         seen.insert(this_og);
-	result.push_back(cur_node);
+        result.push_back(cur_node);
     }
 
-    QPDFObjectHandle type_key = cur_node.getKey("/Type");
-    if (! (type_key.isName() && (type_key.getName() == wanted_type)))
+    if (! cur_node.isDictionaryOfType(wanted_type))
     {
         warn(QPDFExc(qpdf_e_damaged_pdf, this->m->file->getName(),
                      "page tree node",
@@ -345,6 +344,7 @@ QPDF::findPage(QPDFObjGen const& og)
         this->m->pageobj_to_pages_pos.find(og);
     if (it == this->m->pageobj_to_pages_pos.end())
     {
+        QTC::TC("qpdf", "QPDF_pages findPage not found");
         setLastObjectDescription("page object", og.getObj(), og.getGen());
         throw QPDFExc(qpdf_e_pages, this->m->file->getName(),
                       this->m->last_object_description, 0,
