@@ -7,11 +7,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-// See comments in bits.cc
+// See comments in bits_functions.hh
 #define BITS_TESTING 1
 #define BITS_READ 1
 #define BITS_WRITE 1
-#include "../libqpdf/bits.icc"
+#include <qpdf/bits_functions.hh>
 
 static void
 print_values(long long byte_offset, size_t bit_offset,
