@@ -5,9 +5,14 @@
 #include <stdlib.h>
 #include <stdexcept>
 #include <iostream>
-#include <cassert>
 #include <cstring>
 
+#ifdef NDEBUG
+// We need assert even in a release build for test code.
+# undef NDEBUG
+#endif
+#include <cassert>
+
 static unsigned char* uc(char const* s)
 {
     return QUtil::unsigned_char_pointer(s);
