@@ -1,5 +1,4 @@
 #include <iostream>
-#include <cassert>
 #include <cstring>
 #include <cstdlib>
 #include <functional>
@@ -10,6 +9,12 @@
 #include <memory>
 #include <regex>
 
+#ifdef NDEBUG
+// We need assert even in a release build for test code.
+# undef NDEBUG
+#endif
+#include <cassert>
+
 // Functional programming
 
 // Function that returns a callable in the form of a lambda
