@@ -6,10 +6,15 @@
 
 #include <iostream>
 #include <errno.h>
-#include <assert.h>
 #include <string.h>
 #include <stdlib.h>
 
+#ifdef NDEBUG
+// We need assert even in a release build for test code.
+# undef NDEBUG
+#endif
+#include <cassert>
+
 void run(char const* filename, char const* filter,
          bool encode, unsigned int columns,
          unsigned int bits_per_sample, unsigned int samples_per_pixel)
