@@ -110,7 +110,7 @@ QdfFixer::processLines(std::list<std::string>& lines)
     static std::regex re_dict_end("^>>\n$");
 
     lineno = 0;
-    for (auto line: lines)
+    for (auto const& line: lines)
     {
         ++lineno;
         last_offset = offset;
@@ -163,7 +163,7 @@ QdfFixer::processLines(std::list<std::string>& lines)
                 // index. Make sure we get at least 1 byte even if
                 // there are no object streams.
                 int max_objects = 1;
-                for (auto e: xref)
+                for (auto const& e: xref)
                 {
                     if ((e.getType() == 2) &&
                         (e.getObjStreamIndex() > max_objects))
@@ -258,7 +258,7 @@ QdfFixer::processLines(std::list<std::string>& lines)
                 writeBinary(0, 1);
                 writeBinary(0, xref_f1_nbytes);
                 writeBinary(0, xref_f2_nbytes);
-                for (auto x: xref)
+                for (auto const& x: xref)
                 {
                     unsigned long long f1 = 0;
                     unsigned long long f2 = 0;
@@ -294,7 +294,10 @@ QdfFixer::processLines(std::list<std::string>& lines)
         {
             if (matches(re_ignore_newline))
             {
-                --stream_length;
+                if (stream_length > 0)
+                {
+                    --stream_length;
+                }
             }
             else if (matches(re_n_0_obj))
             {
@@ -321,7 +324,7 @@ QdfFixer::processLines(std::list<std::string>& lines)
         {
             auto n = xref.size();
             std::cout << "0 " << 1 + n << "\n0000000000 65535 f \n";
-            for (auto e: xref)
+            for (auto const& e: xref)
             {
                 std::cout << QUtil::int_to_string(e.getOffset(), 10)
                           << " 00000 n \n";
@@ -366,7 +369,7 @@ QdfFixer::checkObjId(std::string const& cur_obj_str)
     int cur_obj = QUtil::string_to_int(cur_obj_str.c_str());
     if (cur_obj != last_obj + 1)
     {
-	fatal(filename + ":" + QUtil::uint_to_string(lineno) +
+        fatal(filename + ":" + QUtil::uint_to_string(lineno) +
               ": expected object " + QUtil::int_to_string(last_obj + 1));
     }
     last_obj = cur_obj;
@@ -389,9 +392,9 @@ QdfFixer::writeOstream()
     auto n = ostream_offsets.size();
     for (auto iter: ostream_offsets)
     {
-	iter -= QIntC::to_offset(first);
-	++onum;
-	offsets += QUtil::int_to_string(onum) + " " +
+        iter -= QIntC::to_offset(first);
+        ++onum;
+        offsets += QUtil::int_to_string(onum) + " " +
             QUtil::int_to_string(iter) + "\n";
     }
     auto offset_adjust = QIntC::to_offset(offsets.size());
@@ -403,21 +406,21 @@ QdfFixer::writeOstream()
     dict_data += "  /First " + QUtil::int_to_string(first) + "\n";
     if (! ostream_extends.empty())
     {
-	dict_data += "  /Extends " + ostream_extends + "\n";
+        dict_data += "  /Extends " + ostream_extends + "\n";
     }
     dict_data += ">>\n";
     offset_adjust += QIntC::to_offset(dict_data.length());
     std::cout << dict_data
               << "stream\n"
               << offsets;
-    for (auto o: ostream)
+    for (auto const& o: ostream)
     {
-	std::cout << o;
+        std::cout << o;
     }
 
-    for (auto o: ostream_discarded)
+    for (auto const& o: ostream_discarded)
     {
-	offset -= QIntC::to_offset(o.length());
+        offset -= QIntC::to_offset(o.length());
     }
     offset += offset_adjust;
 
@@ -445,9 +448,9 @@ QdfFixer::writeBinary(unsigned long long val, size_t bytes)
     }
     for (size_t i = 0; i < bytes; ++i)
     {
-	data.at(bytes - i - 1) =
+        data.at(bytes - i - 1) =
             static_cast<char>(QIntC::to_uchar(val & 0xff));
-	val >>= 8;
+        val >>= 8;
     }
     std::cout << data;
 }
